package sun.awt.util;

import java.util.Arrays;
import java.util.Collection;
import java.util.RandomAccess;
import java.util.List;
import java.util.AbstractList;

public class IdentityArrayList<E> extends AbstractList<E> implements List<E>, RandomAccess
{
    private transient Object[] elementData;
    private int size;
    
    public IdentityArrayList(int i) {
        if (i < 0) {
            throw new IllegalArgumentException("Illegal Capacity: " + i);
        }
        this.elementData = new Object[i];
        return;
    }
    
    public IdentityArrayList() {
        this(10);
    }
    
    public IdentityArrayList(Collection<? extends E> collection) {
        this.elementData = collection.toArray();
        this.size = this.elementData.length;
        if (this.elementData.getClass() != Object[].class) {
            this.elementData = Arrays.copyOf(this.elementData, this.size, Object[].class);
        }
    }
    
    public void trimToSize() {
        ++this.modCount;
        if (this.size < this.elementData.length) {
            this.elementData = Arrays.copyOf(this.elementData, this.size);
        }
    }
    
    public void ensureCapacity(int n) {
        ++this.modCount;
        int length = this.elementData.length;
        if (n > length) {
            Object[] elementData = this.elementData;
            int newLength = length * 3 / 2 + 1;
            if (newLength < n) {
                newLength = n;
            }
            this.elementData = Arrays.copyOf(this.elementData, newLength);
        }
    }
    
    @Override
    public int size() {
        return this.size;
    }
    
    @Override
    public boolean isEmpty() {
        return this.size == 0;
    }
    
    @Override
    public boolean contains(Object o) {
        return this.indexOf(o) >= 0;
    }
    
    @Override
    public int indexOf(Object o) {
        int i = 0;
        while (i < this.size) {
            if (o == this.elementData[i]) {
                return i;
            }
            ++i;
        }
        return -1;
    }
    
    @Override
    public int lastIndexOf(Object o) {
        int i = this.size - 1;
        while (i >= 0) {
            if (o == this.elementData[i]) {
                return i;
            }
            --i;
        }
        return -1;
    }
    
    @Override
    public Object[] toArray() {
        return Arrays.copyOf(this.elementData, this.size);
    }
    
    @Override
    public <T> T[] toArray(T[] array) {
        if (array.length < this.size) {
            return (T[])Arrays.copyOf(this.elementData, this.size, array.getClass());
        }
        System.arraycopy(this.elementData, 0, array, 0, this.size);
        if (array.length > this.size) {
            array[this.size] = null;
            return array;
        }
        return array;
    }
    
    @Override
    public E get(int n) {
        this.rangeCheck(n);
        return this.elementData[n];
    }
    
    @Override
    public E set(int n, E e) {
        this.rangeCheck(n);
        Object o = this.elementData[n];
        this.elementData[n] = e;
        return o;
    }
    
    @Override
    public boolean add(E e) {
        this.ensureCapacity(this.size + 1);
        this.elementData[this.size++] = e;
        return true;
    }
    
    @Override
    public void add(int n, E e) {
        this.rangeCheckForAdd(n);
        this.ensureCapacity(this.size + 1);
        System.arraycopy(this.elementData, n, this.elementData, n + 1, this.size - n);
        this.elementData[n] = e;
        ++this.size;
    }
    
    @Override
    public E remove(int n) {
        this.rangeCheck(n);
        ++this.modCount;
        Object o = this.elementData[n];
        int n2 = this.size - n - 1;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n + 1, this.elementData, n, n2);
        }
        this.elementData[--this.size] = null;
        return o;
    }
    
    @Override
    public boolean remove(Object o) {
        int i = 0;
        while (i < this.size) {
            if (o == this.elementData[i]) {
                this.fastRemove(i);
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private void fastRemove(int n) {
        ++this.modCount;
        int n2 = this.size - n - 1;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n + 1, this.elementData, n, n2);
        }
        this.elementData[--this.size] = null;
    }
    
    @Override
    public void clear() {
        ++this.modCount;
        for (int i = 0; i < this.size; ++i) {
            this.elementData[i] = null;
        }
        this.size = 0;
    }
    
    @Override
    public boolean addAll(Collection<? extends E> collection) {
        Object[] array = collection.toArray();
        int length = array.length;
        this.ensureCapacity(this.size + length);
        System.arraycopy(array, 0, this.elementData, this.size, length);
        this.size += length;
        return length != 0;
    }
    
    @Override
    public boolean addAll(int n, Collection<? extends E> collection) {
        this.rangeCheckForAdd(n);
        Object[] array = collection.toArray();
        int length = array.length;
        this.ensureCapacity(this.size + length);
        int n2 = this.size - n;
        if (n2 > 0) {
            System.arraycopy(this.elementData, n, this.elementData, n + length, n2);
        }
        System.arraycopy(array, 0, this.elementData, n, length);
        this.size += length;
        return length != 0;
    }
    
    @Override
    protected void removeRange(int n, int n2) {
        ++this.modCount;
        System.arraycopy(this.elementData, n2, this.elementData, n, this.size - n2);
        while (this.size != this.size - (n2 - n)) {
            this.elementData[--this.size] = null;
        }
    }
    
    private void rangeCheck(int n) {
        if (n >= this.size) {
            throw new IndexOutOfBoundsException(this.outOfBoundsMsg(n));
        }
        return;
    }
    
    private void rangeCheckForAdd(int n) {
        if (n > this.size || n < 0) {
            throw new IndexOutOfBoundsException(this.outOfBoundsMsg(n));
        }
        return;
    }
    
    private String outOfBoundsMsg(int i) {
        return "Index: " + i + ", Size: " + this.size;
    }
}
